#include "StdAfx.h"
#include "moebiusstrip.h"

MoebiusStrip::MoebiusStrip(void)
{
 SideNum=1;
 Turns=1;
 Radius=5;
 Width=1;
 Height=.1f;

 StepNum=36;
}

MoebiusStrip::~MoebiusStrip(void)
{
}

void MoebiusStrip::Clear()
{
	Ring.Clear();
}
void MoebiusStrip::Generate()
{
	const float BlockSide=3;
	int StepNum=36;
	float StepAngleDeg=360.0f/StepNum;
	float StepAngleRad=2.0f*3.14159265f/StepNum;

	double MM[16],MP[16];
	

	//int OldViewPort[4];
	//glGetIntegerv(GL_VIEWPORT,OldViewPort);
	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();
	glOrtho(-1,1,-1,1,-1,1);
	glGetDoublev(GL_PROJECTION_MATRIX,MP);

	// Inizializzazione della Modelview Matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();	
  	
	// -- Disegno dell'Anello -- 
	// l'anello viene create simmetrico rispetto all'asse z.
	// Per renderlo di moebius basta fare aumentare la rotazione lungo
	// l'asse y, fatta prima di ruotare intorno all'asse z; Tale rotazione 
  // deve essere la meta' di quella intorno all'asse z
	// (cosi' dopo 360 gradi ha fatto 180 gradi)
	
	// Per far si'che il calcolo delle normali interpolate sia corretto si 
	// aggiunge due strisce di triangoli in piu' che vengono tolte alla fine. 
  // alla fine si togliera quindi 4 triangoli all'inizio e quattro alla fine
	Point3d nn(0,0,0);
	for(int i=0;i<StepNum+3;++i)
	{
		glPushMatrix();
		glRotatef(i*StepAngleDeg    , 0,0,1);   // Rotazione Intorno all'asse principale
		glTranslatef(Radius,0,0);
		glRotatef(i*StepAngleDeg/2.0f, 0,1,0);   // Avvolgimento della striscia.
	
		// -- Disegno del blocchetto dell'anello -- 
		Point3d pp;
		glGetDoublev(GL_MODELVIEW_MATRIX,MM);
		int vp[4]={-1,-1,2,2};
		float hh=Height/2.0f;
		float hw=Width/2.0f;

		gluProject(-hw,0,hh,MM,MP,vp,&pp[0],&pp[1],&pp[2]);
		pp[2]=-2.0f*(pp[2]-.5);
		Ring.vert.push_back(CIVertex(pp,nn));
		
		gluProject(hw,0,hh,MM,MP,vp,&pp[0],&pp[1],&pp[2]);
		pp[2]=-2.0f*(pp[2]-.5);
		Ring.vert.push_back(CIVertex(pp,nn));
		
		gluProject(hw,0,-hh,MM,MP,vp,&pp[0],&pp[1],&pp[2]);
		pp[2]=-2.0f*(pp[2]-.5);
		Ring.vert.push_back(CIVertex(pp,nn));
		
		gluProject(-hw,0,-hh,MM,MP,vp,&pp[0],&pp[1],&pp[2]);
		pp[2]=-2.0f*(pp[2]-.5);
		Ring.vert.push_back(CIVertex(pp,nn));
		
		glPopMatrix();
	}
  //  a--c
	//  |\ |
	//  | \|
	//  b--d  
	//


	for(int i=0;i<StepNum+2;++i)
	{
		for(int k=0;k<4;++k)
		{
			CIFace tmp;

			tmp.v[0]=&(Ring.vert[(i+0)*4+(k+0)%4]); // a 
			tmp.v[1]=&(Ring.vert[(i+0)*4+(k+1)%4]); // b 
			tmp.v[2]=&(Ring.vert[(i+1)*4+(k+1)%4]); // d 
			Ring.face.push_back(tmp);

			tmp.v[0]=&(Ring.vert[(i+0)*4+(k+0)%4]); // a
			tmp.v[1]=&(Ring.vert[(i+1)*4+(k+1)%4]); // d
			tmp.v[2]=&(Ring.vert[(i+1)*4+(k+0)%4]); // c
			Ring.face.push_back(tmp);
		}
	}

	Ring.ComputeFaceNormal();
	Ring.ComputeVertNormal();

  // cancello la prima e l'ultima striscia di facce che avevo 
	// messo solo per calcolare in maniera corretta le normali per vertice.
	// Nota ora ci sono in Ring.vert alcuni vertici che non sono riferiti 
	// da alcuna faccia
	Ring.face.erase(Ring.face.begin(),Ring.face.begin()+8);
	Ring.face.erase(Ring.face.end()-8,Ring.face.end());
}
